import React from 'react';
import PropTypes from 'prop-types';
import { createComponent } from 'cf-style-container';

const radius = (borderRadius, group, spaced) => {
  if (spaced) {
    return { borderRadius };
  }
  if (group && !spaced) {
    return {
      borderTopRightRadius: group === 'last' ? borderRadius : 0,
      borderTopLeftRadius: group === 'first' ? borderRadius : 0,
      borderBottomLeftRadius: group === 'first' ? borderRadius : 0,
      borderBottomRightRadius: group === 'last' ? borderRadius : 0
    };
  }
  return { borderRadius };
};

const opacity = (loading, disabled) => {
  if (loading) {
    return { opacity: 0.8 };
  }
  if (disabled) {
    return { opacity: 0.5 };
  }
  return { opacity: 'inherit' };
};

const before = (fadeZoomIn, loading) => {
  if (!loading) {
    return null;
  }
  // this needs something better (e.g. a rotating icon)
  return {
    '&::before': {
      content: '"…"',
      color: 'black',
      display: 'block',
      position: 'absolute',
      top: '0.6em',
      left: 0,
      height: '100%',
      width: '100%',
      opacity: 1,
      animationName: {
        '50%': { opacity: 0 }
      },
      animationDuration: '1s',
      animationIterationCount: 'infinite',
      animationTimingFunction: 'linear'
    }
  };
};

const marginLeft = (marginLeft, group, spaced, direction) => {
  if (!group || direction === 'column') {
    return { marginLeft };
  }
  if (spaced && group !== 'first') {
    return { marginLeft: '0.4rem' };
  }
  return { marginLeft: 0 };
};

const marginTop = (marginTop, group, spaced, direction) => {
  if (!group || direction === 'row') {
    return { marginTop };
  }
  if (spaced && direction === 'column' && group !== 'first') {
    return { marginTop: '0.4rem' };
  }
  return { marginTop: 0 };
};

const border = (theme, group, type, loading) => {
  const isOutlineButton = type === 'dangerOutline';
  const borderColor = loading ? theme.disabledBorder : theme[`${type}Border`];

  return Object.assign(
    isOutlineButton
      ? {
          borderBottom: '1px solid',
          borderLeft: '1px solid',
          borderRight: '1px solid',
          borderTop: '1px solid'
        }
      : {
          borderBottom: theme.borderBottom,
          borderLeft: group && group !== 'first' ? 0 : theme.borderLeft,
          borderRight: theme.borderRight,
          borderTop: theme.borderTop
        },
    {
      borderColor: borderColor
    }
  );
};

const styles = props => {
  const theme = props.theme;
  return {
    '&:hover': {
      backgroundColor: props.loading
        ? theme.disabledBackground
        : theme[`${props.type}HoverBackground`],
      borderColor: props.loading
        ? theme.disabledBorder
        : theme[`${props.type}HoverBorder`],
      color: props.loading
        ? theme.disabledBackground
        : theme[`${props.type}HoverColor`]
    },
    '&:active': {
      backgroundColor: theme[`${props.type}ActiveBackground`],
      borderColor: props.loading
        ? theme.disabledBorder
        : theme[`${props.type}ActiveBorder`],
      color: theme[`${props.type}ActiveColor`]
    },
    '&:focus': {
      backgroundColor: props.loading
        ? theme.backgroundColor
        : theme[`${props.type}FocusBackground`],
      boxShadow: `inset 0px 0px 0px ${theme.borderSize} ${theme[`${props.type}focusOutlineColor`]}`,
      color: props.loading
        ? theme.disabledBackground
        : theme[`${props.type}FocusColor`],
      outline: props.loading ? 'none' : undefined,
      borderColor: props.loading
        ? theme.disabledBorder
        : theme[`${props.type}Border`]
    },
    ...before(theme.fadeZoomIn, props.loading),
    '&[title]': {
      pointerEvents: props.disabled ? 'auto' : 'none'
    },
    backgroundColor: props.loading
      ? theme.disabledBackground
      : theme[`${props.type}Background`],
    ...border(theme, props.group, props.type, props.loading),
    color: props.loading
      ? theme.disabledBackground
      : theme[`${props.type}Color`],
    cursor: props.disabled || props.loading ? 'default' : theme.cursor,
    display: theme.display,
    fontFamily: theme.fontFamily,
    fontSize: theme.fontSize,
    fontWeight: theme.fontWeight,
    lineHeight: theme.lineHeight,
    marginBottom: props.group ? 0 : theme.marginBottom,
    marginRight: props.group ? 0 : theme.marginRight,
    marginTop: props.group ? 0 : theme.marginTop,
    ...marginTop(theme.marginTop, props.group, props.spaced, props.direction),
    ...marginLeft(theme.marginLeft, props.group, props.spaced, props.direction),
    ...opacity(props.loading, props.disabled),
    paddingBottom: theme.paddingBottom,
    paddingLeft: theme.paddingLeft,
    paddingRight: theme.paddingRight,
    paddingTop: theme.paddingTop,
    pointerEvents: props.disabled ? 'none' : 'auto',
    position: theme.position,
    ...radius(theme.borderRadius, props.group, props.spaced),
    textAlign: theme.textAlign,
    transition: theme.transition,
    userSelect: theme.userSelect,
    float: theme.float,
    maxWidth: theme.maxWidth,
    float: theme.float,
    width: props.direction === 'column' ? '100%' : 'auto'
  };
};

class Button extends React.Component {
  render() {
    const {
      onClick,
      submit,
      className,
      disabled,
      loading,
      children
    } = this.props;
    return (
      <button
        type={submit ? 'submit' : 'button'}
        disabled={disabled || loading}
        className={className}
        onClick={onClick}
      >
        {children}
      </button>
    );
  }
}

Button.propTypes = {
  onClick: PropTypes.func,
  submit: PropTypes.bool,
  spaced: PropTypes.bool,
  direction: PropTypes.oneOf(['column', 'row']),
  className: PropTypes.string.isRequired,
  group: PropTypes.oneOf(['first', 'inbetween', 'last']),
  type: PropTypes.oneOf([
    'default',
    'primary',
    'success',
    'warning',
    'danger',
    'dangerOutline'
  ]).isRequired,
  disabled: PropTypes.bool,
  loading: PropTypes.bool,
  children: PropTypes.node
};

Button.defaultProps = {
  type: 'default',
  submit: false,
  loading: false
};

Button.displayName = 'Button';

export default createComponent(styles, Button);
